@@ -1,18 +1,13 @@
-# $Id: Error.pm,v 1.29 2003/12/23 06:22:46 petdance Exp $
 package HTML::Lint::Error;
 
 use strict;
 
-use Exporter;
+use base 'Exporter';
+our @EXPORT = ();
+our @EXPORT_OK = qw( STRUCTURE HELPER FLUFF );
+our %EXPORT_TAGS = ( types => [@EXPORT_OK] );
 
-use vars qw( @ISA @EXPORT @EXPORT_OK %EXPORT_TAGS );
-@ISA = 'Exporter';
-
-@EXPORT = ();
-@EXPORT_OK = qw( STRUCTURE HELPER FLUFF );
-%EXPORT_TAGS = ( types => [@EXPORT_OK] ); 
- 
-use vars qw( %errors );
+our %errors;
 
 =head1 NAME
 
@@ -58,9 +53,9 @@ This is usually something like an unknown attribute on a tag.
 
 =cut
 
-use constant STRUCTURE	=> 1;
-use constant HELPER	=> 2;
-use constant FLUFF	=> 3;
+use constant STRUCTURE  => 1;
+use constant HELPER     => 2;
+use constant FLUFF      => 3;
 
 =head2 new()
 
@@ -80,12 +75,12 @@ sub new {
     # Add an element that says what tag caused the error (B, TR, etc)
     # so that we can match 'em up down the road.
     my $self  = {
-	_file => $file,
-	_line => $line,
-	_column => $column,
-	_errcode => $errcode,
-	_errtext => undef,
-	_type => undef,
+        _file => $file,
+        _line => $line,
+        _column => $column,
+        _errcode => $errcode,
+        _errtext => undef,
+        _type => undef,
     };
 
     bless $self, $class;
@@ -99,19 +94,20 @@ sub _expand_error {
     my $self = shift;
 
     my $errcode = shift;
-    
+
     my $specs = $errors{$errcode};
     my $str;
     if ( $specs ) {
-	($str, $self->{_type}) = @$specs;
-    } else {
-	$str = "Unknown code: $errcode";
+        ($str, $self->{_type}) = @$specs;
+    }
+    else {
+        $str = "Unknown code: $errcode";
     }
 
     while ( @_ ) {
-	my $var = shift;
-	my $val = shift;
-	$str =~ s/\$\{$var\}/$val/g;
+        my $var = shift;
+        my $val = shift;
+        $str =~ s/\$\{$var\}/$val/g;
     }
 
     $self->{_errtext} = $str;
@@ -130,7 +126,7 @@ sub is_type {
     my $self = shift;
 
     for my $matcher ( @_ ) {
-	return $matcher if $matcher eq $self->type;
+        return $matcher if $matcher eq $self->type;
     }
 
     return;
@@ -166,12 +162,12 @@ sub where {
     my $col;
 
     if ( not ref $_[0] ) {
-	$line = shift;
-	$col = shift;
+        $line = shift;
+        $col = shift;
     } else {
-	my $self = shift;
-	$line = $self->line;
-	$col = $self->column;
+        my $self = shift;
+        $line = $self->line;
+        $col = $self->column;
     }
     $col ||= 0;
     return sprintf( "(%s:%s)", $line, $col + 1 );
@@ -215,12 +211,12 @@ Type of the error
 
 =cut
 
-sub file 	{ my $self = shift; return $self->{_file} 	|| '' }
-sub line 	{ my $self = shift; return $self->{_line} 	|| '' }
-sub column 	{ my $self = shift; return $self->{_column} 	|| '' }
-sub errcode 	{ my $self = shift; return $self->{_errcode} 	|| '' }
-sub errtext 	{ my $self = shift; return $self->{_errtext} 	|| '' }
-sub type 	{ my $self = shift; return $self->{_type} 	|| '' }
+sub file        { my $self = shift; return $self->{_file}       || '' }
+sub line        { my $self = shift; return $self->{_line}       || '' }
+sub column      { my $self = shift; return $self->{_column}     || '' }
+sub errcode     { my $self = shift; return $self->{_errcode}    || '' }
+sub errtext     { my $self = shift; return $self->{_errtext}    || '' }
+sub type        { my $self = shift; return $self->{_type}       || '' }
 
 
 =head1 TODO
@@ -236,7 +232,7 @@ employers of the various contributors to the code.
 
 =head1 AUTHOR
 
-Andy Lester, E<lt>andy@petdance.comE<gt>
+Andy Lester, C<andy at petdance.com>
 
 =cut
 
@@ -245,21 +241,21 @@ Andy Lester, E<lt>andy@petdance.comE<gt>
 
 # Generic element stuff
 %errors = (
-    'elem-unknown' =>		['Unknown element <${tag}>', STRUCTURE],
-    'elem-unopened' =>		['</${tag}> with no opening <${tag}>', STRUCTURE],
-    'elem-unclosed' =>		['<${tag}> at ${where} is never closed', STRUCTURE],
-    'elem-empty-but-closed' =>	['<${tag}> is not a container -- </${tag}> is not allowed', STRUCTURE],
+    'elem-unknown' =>           ['Unknown element <${tag}>', STRUCTURE],
+    'elem-unopened' =>          ['</${tag}> with no opening <${tag}>', STRUCTURE],
+    'elem-unclosed' =>          ['<${tag}> at ${where} is never closed', STRUCTURE],
+    'elem-empty-but-closed' =>  ['<${tag}> is not a container -- </${tag}> is not allowed', STRUCTURE],
 
-    'elem-img-sizes-missing' =>	['<IMG> tag has no HEIGHT and WIDTH attributes.', HELPER],
+    'elem-img-sizes-missing' => ['<IMG> tag has no HEIGHT and WIDTH attributes.', HELPER],
     'elem-img-alt-missing' =>   ['<IMG> does not have ALT text defined', HELPER],
-    'elem-nonrepeatable' =>	['<${tag}> is not repeatable, but already appeared at ${where}', STRUCTURE],
+    'elem-nonrepeatable' =>     ['<${tag}> is not repeatable, but already appeared at ${where}', STRUCTURE],
 
-    'doc-tag-required' =>	['<${tag}> tag is required', STRUCTURE],
+    'doc-tag-required' =>       ['<${tag}> tag is required', STRUCTURE],
 
-    'attr-repeated' =>		['${attr} attribute in <${tag}> is repeated', STRUCTURE],
-    'attr-unknown' =>		['Unknown attribute "${attr}" for tag <${tag}>', FLUFF],
+    'attr-repeated' =>          ['${attr} attribute in <${tag}> is repeated', STRUCTURE],
+    'attr-unknown' =>           ['Unknown attribute "${attr}" for tag <${tag}>', FLUFF],
 
-    'text-use-entity' =>	['Invalid character ${char} should be written as ${entity}', STRUCTURE],
+    'text-use-entity' =>        ['Invalid character ${char} should be written as ${entity}', STRUCTURE],
 );
 
 1; # happy
@@ -267,27 +263,27 @@ Andy Lester, E<lt>andy@petdance.comE<gt>
 __DATA__
 Errors that haven't been done yet.
 
-#elem-head-only			<${tag}> can only appear in the <HEAD> element
-#elem-non-head-element 		<${tag}> cannot appear in the <HEAD> element
-#elem-obsolete 			<${tag}> is obsolete
-#elem-nested-element 		<${tag}> cannot be nested -- one is already opened at ${where}
-#elem-wrong-context		Illegal context for <${tag}> -- must appear in <${othertag}> tag.
-#elem-heading-in-anchor		<A> should be inside <${tag}>, not <${tag}> inside <A>
+#elem-head-only                 <${tag}> can only appear in the <HEAD> element
+#elem-non-head-element          <${tag}> cannot appear in the <HEAD> element
+#elem-obsolete                  <${tag}> is obsolete
+#elem-nested-element            <${tag}> cannot be nested -- one is already opened at ${where}
+#elem-wrong-context             Illegal context for <${tag}> -- must appear in <${othertag}> tag.
+#elem-heading-in-anchor         <A> should be inside <${tag}>, not <${tag}> inside <A>
 
-#elem-head-missing		No <HEAD> element found
-#elem-head-missing-title 	No <TITLE> in <HEAD> element
-#elem-img-sizes-incorrect	<IMG> tag's HEIGHT and WIDTH attributes are incorrect.  They should be ${correct}.
-#attr-missing 			<${tag}> is missing a "${attr}" attribute
+#elem-head-missing              No <HEAD> element found
+#elem-head-missing-title        No <TITLE> in <HEAD> element
+#elem-img-sizes-incorrect       <IMG> tag's HEIGHT and WIDTH attributes are incorrect.  They should be ${correct}.
+#attr-missing                   <${tag}> is missing a "${attr}" attribute
 
-#comment-unclosed		Unclosed comment
-#comment-markup			Markup embedded in a comment can confuse some browsers
+#comment-unclosed               Unclosed comment
+#comment-markup                 Markup embedded in a comment can confuse some browsers
 
-#text-literal-metacharacter	Metacharacter $char should be represented as "$otherchar"
-#text-title-length		The HTML spec recommends that that <TITLE> be no more than 64 characters
-#text-markup			Tag <${tag}> found in the <TITLE>, which will not be rendered properly.
+#text-literal-metacharacter     Metacharacter $char should be represented as "$otherchar"
+#text-title-length              The HTML spec recommends that that <TITLE> be no more than 64 characters
+#text-markup                    Tag <${tag}> found in the <TITLE>, which will not be rendered properly.
 
-#elem-physical-markup		<${tag}> is physical font markup.  Use logical (such as <${othertag}>) instead.
-#elem-leading-whitespace	<${tag}> should not have whitespace between "<" and "${tag}>"
+#elem-physical-markup           <${tag}> is physical font markup.  Use logical (such as <${othertag}>) instead.
+#elem-leading-whitespace        <${tag}> should not have whitespace between "<" and "${tag}>"
 #'must-follow' => [ ENABLED, MC_ERROR, '<$argv[0]> must immediately follow <$argv[1]>', ],
 # 'empty-container' => [ ENABLED, MC_WARNING, 'empty container element <$argv[0]>.', ],
 # 'directory-index' => [ ENABLED, MC_WARNING, 'directory $argv[0] does not have an index file ($argv[1])', ],
